-------------------------------------------------------------------------------
-- ElvUI Raid Markers Bar By Lockslap
-- <Borderline Amazing> US, Bleeding Hollow
-- Contains modifications graciously provided by Dridzt!
-------------------------------------------------------------------------------
local _G = getfenv()
local E, _, V, P, G, _ = unpack(ElvUI)
local RM = E:NewModule("RaidMarkers")
local L = LibStub("AceLocale-3.0"):GetLocale("ElvUI_RaidMarkers", false)
local M = E:GetModule("Minimap")
local EP = LibStub("LibElvUIPlugin-1.0")

local BUTTON_HEIGHT = 18
local BUTTON_WIDTH = 18
local BUTTON_DISTANCE = 5
local FRAME_HEIGHT = 22
local FRAME_WIDTH = 150

local COMBAT_QUEUE = {}
local MODIFIER_DEFAULT = "shift-"

local inCombat
local buttonMap = {
	[1] = {RT = 1, WM = 5},	-- yellow/star
	[2] = {RT = 2},			-- orange/circle
	[3] = {RT = 3, WM = 3},	-- purple/diamond
	[4] = {RT = 4, WM = 2},	-- green/triangle
	[5] = {RT = 5},			-- white/moon
	[6] = {RT = 6, WM = 1},	-- blue/square
	[7] = {RT = 7, WM = 4},	-- red/cross
	[8] = {RT = 8},			-- white/skull
	[9] = {RT = 0, WM = 0},	-- clear target/flare
}

local function Capitalize(str)
	return str:gsub("^%l", string.upper)
end

local function InCombat()
	return inCombat or InCombatLockdown() or UnitAffectingCombat("player") or UnitAffectingCombat("pet")
end

local function ButtonFactory()
	-- our combat check
	if InCombat() then
		if not tContains(COMBAT_QUEUE, ButtonFactory) then
			tinsert(COMBAT_QUEUE, ButtonFactory)
		end
		return
	end
	
	local bar = _G["ElvRMFrame"]
	for i, buttonData in ipairs(buttonMap) do
		local button = CreateFrame("Button", ("ElvRMFrameButton%d"):format(i), bar, "SecureActionButtonTemplate")
		button:SetHeight(BUTTON_HEIGHT)
		button:SetWidth(BUTTON_WIDTH)
		
		local image = button:CreateTexture(nil, "BACKGROUND")
		image:SetAllPoints()
		if i == 9 then
			image:SetTexture("Interface\\BUTTONS\\UI-GroupLoot-Pass-Up")
		else
			image:SetTexture(("Interface\\TargetingFrame\\UI-RaidTargetingIcon_%d"):format(i))
		end
		
		local highlight = button:CreateTexture(nil, "HIGHLIGHT")
		highlight:SetAllPoints(image)
		highlight:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-Tab-Highlight")
		highlight:SetTexCoord(0, 1, 0.23, 0.77)
		highlight:SetBlendMode("ADD")
		
		bar[("button%d"):format(i)] = button
		
		local target, flare = buttonData.RT, buttonData.WM
		-- target markers
		if target then
			button:SetAttribute("type1", "macro")
			button:SetAttribute("macrotext1", ("/run SetRaidTargetIcon(\"target\", %d)"):format(i))
			
			-- tooltip
			button:SetScript("OnEnter", function(self)
				GameTooltip:SetOwner(self, "ANCHOR_BOTTOM")
				GameTooltip:SetText(L["ElvUI Raid Markers"])
				GameTooltip:AddLine(i == 9 and L["Click to clear the mark."] or L["Click to mark the target."], 1, 1, 1)
				GameTooltip:Show()
			end)
			button:SetScript("OnLeave", function() GameTooltip:Hide() end)
		end
		
		-- flares
		if flare then
			-- emphasis for flares
			local overlay = button:CreateTexture(nil, "OVERLAY")
			overlay:SetAllPoints(highlight)
			overlay:SetTexture("Interface\\BUTTONS\\CheckButtonHilight")
			overlay:SetAlpha(0.35)
			overlay:SetBlendMode("ADD")
			
			-- flare access
			local modifier = E.db.actionbar.raidmarkers.modifier or MODIFIER_DEFAULT
			button:SetAttribute(("%stype1"):format(modifier), "macro")
			button.modifier = modifier
			if flare == 0 then
				button:SetAttribute(("%smacrotext1"):format(modifier), "/clearworldmarker all")
			else
				button:SetAttribute(("%smacrotext1"):format(modifier), ("/clearworldmarker %d\n/worldmarker %d"):format(flare, flare))
			end
			
			-- tooltip
			button:SetScript("OnEnter", function(self)
				GameTooltip:SetOwner(self, "ANCHOR_BOTTOM")
				GameTooltip:SetText(L["ElvUI Raid Markers"])
				GameTooltip:AddLine(i == 9 and (L["Click to clear the mark.\n%sClick to remove all flares."]):format(Capitalize(button.modifier)) or (L["Click to mark the target.\n%sClick to place a flare."]):format(Capitalize(button.modifier)), 1, 1, 1)
				GameTooltip:Show()
			end)
			button:SetScript("OnLeave", function() GameTooltip:Hide() end)
		end
		button:RegisterForClicks("AnyDown")
	end
end

local function ButtonModifierUpdate()
	if InCombat() then
		if not tContains(COMBAT_QUEUE, ButtonModifierUpdate) then
			tinsert(COMBAT_QUEUE, ButtonModifierUpdate)
		end
		return
	end
	
	local prev = _G["ElvRMFrame"].button1.modifier
	local newMod = E.db.actionbar.raidmarkers.modifier
	if newMod and newMod ~= prev then
		for i, buttonData in ipairs(buttonMap) do
			if buttonData.WM then
				local button = _G["ElvRMFrame"][("button%d"):format(i)]
				button:SetAttribute(("%stype1"):format(button.modifier), nil)
				button:SetAttribute(("%stype1"):format(newMod), "macro")
				local value = button:GetAttribute(("%smacrotext1"):format(button.modifier))
				button:SetAttribute(("%smacrotext1"):format(button.modifier), nil)
				button:SetAttribute(("%smacrotext1"):format(newMod), value)
				button.modifier = newMod
			end
		end
	end
end

local function ToggleBar()
	if InCombat() then
		if not tContains(COMBAT_QUEUE, ToggleBar) then
			tinsert(COMBAT_QUEUE, ToggleBar)
		end
		return
	end
	
	if E.db.actionbar.raidmarkers.show and not _G["ElvRMFrame"]:IsShown() then
		_G["ElvRMFrame"]:Show()
	elseif not E.db.actionbar.raidmarkers.show and _G["ElvRMFrame"]:IsShown() then
		_G["ElvRMFrame"]:Hide()
	end
end

local function UpdateBar()
	if InCombat() then
		if not tContains(COMBAT_QUEUE, UpdateBar) then
			tinsert(COMBAT_QUEUE, UpdateBar)
		end
		return
	end

	local cToggle = ElvConfigToggle:GetWidth() or 0
	FRAME_WIDTH = Minimap:GetWidth() + cToggle + 1
	BUTTON_DISTANCE = math.ceil((FRAME_WIDTH - (BUTTON_WIDTH * 9)) / 9)
	_G["ElvRMFrame"]:SetWidth(FRAME_WIDTH)
	_G["ElvRMFrame"]:ClearAllPoints()
	_G["ElvRMFrame"]:SetPoint("TOPLEFT", LeftMiniPanel:IsShown() and LeftMiniPanel or Minimap, "BOTTOMLEFT", LeftMiniPanel:IsShown() and 0 or -1, 1)
	for i = 9, 1, -1 do
		local button = _G[("ElvRMFrameButton%d"):format(i)]
		local prev = _G[("ElvRMFrameButton%d"):format(i + 1)]
		button:ClearAllPoints()
		if i == 9 then
			button:SetPoint("LEFT", 3, 0)
		else
			button:SetPoint("LEFT", prev, "RIGHT", BUTTON_DISTANCE, 0)
		end
	end
	
	-- toggle visibility
	local bar = _G["ElvRMFrame"]
	if E.db.actionbar.raidmarkers.auto and not bar.auto then
		RegisterStateDriver(bar, "visibility", "[noexists,nogroup]hide;show")
		bar.auto = true
	elseif not E.db.actionbar.raidmarkers.auto and bar.auto then
		UnregisterStateDriver(bar, "visibility")
		bar.auto = nil
	end
	
	ToggleBar()
end

function RM:Initialize()
	-- figure out the distance between the icons
	local cToggle = ElvConfigToggle:GetWidth() or 0
	FRAME_WIDTH = Minimap:GetWidth() + cToggle + 1
	BUTTON_DISTANCE = math.ceil((FRAME_WIDTH - (BUTTON_WIDTH * 9)) / 9)
	
	local f = CreateFrame("Frame", "ElvRMFrame", E.UIParent, "SecureHandlerStateTemplate")
	f:SetResizable(false)
	f:SetHeight(FRAME_HEIGHT)
	f:SetWidth(FRAME_WIDTH)
	f:SetTemplate("Default", true)
	f:SetPoint("TOPLEFT", LeftMiniPanel:IsShown() and LeftMiniPanel or Minimap, "BOTTOMLEFT", LeftMiniPanel:IsShown() and 0 or -1, 1)
	
	-- create the buttons
	ButtonFactory()
	hooksecurefunc(M, "UpdateSettings", UpdateBar)
end

E:RegisterModule(RM:GetName())

P["actionbar"]["raidmarkers"] = {
	["show"] = true,
	["modifier"] = "shift-",
	["auto"] = true,
}

local function InjectOptions()
	E.Options.args.actionbar.args.raidmarkers = {
		order = -1,
		type = "group",
		name = L["Raid Markers"],
		args = {
			show = {
				type = "toggle",
				order = 1,
				name = L["Show Raid Markers Bar"],
				desc = L["Toggle the raid markers bar."],
				get = function() return E.db.actionbar.raidmarkers.show end,
				set = function(_, value) E.db.actionbar.raidmarkers.show = value; ToggleBar(); end,
			},
			auto = {
				type = "toggle",
				order = 2,
				name = L["Auto Show/Hide"],
				desc = L["Automatically toggle the marker bar based on your target.\n\n|cff00ff00This only applies when you are solo.|r"],
				get = function() return E.db.actionbar.raidmarkers.auto end,
				set = function(_, value) E.db.actionbar.raidmarkers.auto = value; UpdateBar(); end,
			},
			modifier = {
				type = "select",
				order = 3,
				name = L["World Markers Modifier"],
				desc = L["Choose the button modifier to use the world markers (flares)."],
				values = {
					["alt-"] = L["Alt"],
					["ctrl-"] = L["Control"],
					["shift-"] = L["Shift"],
				},
				get = function() return E.db.actionbar.raidmarkers.modifier end,
				set = function(_, value) E.db.actionbar.raidmarkers.modifier = value; ButtonModifierUpdate(); end,
			},
		},
	}
end

EP:RegisterPlugin(..., InjectOptions)
	
SlashCmdList["ELVRAIDMARKERS"] = function(msg, editbox)
	E.db.actionbar.raidmarkers.show = not E.db.actionbar.raidmarkers.show
	ToggleBar()
end
SLASH_ELVRAIDMARKERS1 = "/erm"

-- for the combat queue
local f = CreateFrame("Frame")
f:RegisterEvent("PLAYER_REGEN_DISABLED")
f:RegisterEvent("PLAYER_REGEN_ENABLED")
f:SetScript("OnEvent", function(self, event, ...)
	if event == "PLAYER_REGEN_DISABLED" then
		inCombat = true
	elseif event == "PLAYER_REGEN_ENABLED" then
		-- out of combat, clear the queue
		inCombat = nil
		while next(COMBAT_QUEUE) do
			local func = tremove(COMBAT_QUEUE, 1)
			func()
		end
	end
end)